@@ -1131,6 +1131,7 @@ public:
 
         if (foundIndexInBucket) {
             // 'request' is already present, return the existing index
+            Q_ASSERT(m_currentBucket < m_buckets.size());
             return createIndex(lastBucketWalked, foundIndexInBucket);
         }
 
@@ -1248,17 +1249,17 @@ public:
                     //Create a new monster-bucket at the end of the data
                     int needMonsterExtent = (totalSize - ItemRepositoryBucketSize) / MyBucket::DataSize + 1;
                     Q_ASSERT(needMonsterExtent);
-                    if (m_currentBucket + needMonsterExtent + 1 > m_buckets.size()) {
-                        m_buckets.resize(m_buckets.size() + 10 + needMonsterExtent + 1);
+                    const auto currentBucketIncrease = needMonsterExtent + 1;
+                    if (m_currentBucket + currentBucketIncrease >= m_buckets.size()) {
+                        m_buckets.resize(m_buckets.size() + 10 + currentBucketIncrease);
                     }
                     useBucket = m_currentBucket;
 
                     convertMonsterBucket(useBucket, needMonsterExtent);
-                    m_currentBucket += 1 + needMonsterExtent;
+                    m_currentBucket += currentBucketIncrease;
                     Q_ASSERT(m_currentBucket < ItemRepositoryBucketLimit);
-                    Q_ASSERT(m_buckets[m_currentBucket - 1 - needMonsterExtent] &&
-                             m_buckets[m_currentBucket - 1 - needMonsterExtent]->monsterBucketExtent() ==
-                             needMonsterExtent);
+                    Q_ASSERT(m_buckets[useBucket]);
+                    Q_ASSERT(m_buckets[useBucket]->monsterBucketExtent() == needMonsterExtent);
                 }
                 Q_ASSERT(useBucket);
                 bucketPtr = bucketForIndex(useBucket);
@@ -1345,6 +1346,7 @@ public:
                 if (reOrderFreeSpaceBucketIndex != -1)
                     updateFreeSpaceOrder(reOrderFreeSpaceBucketIndex);
 
+                Q_ASSERT(m_currentBucket < m_buckets.size());
                 return createIndex(useBucket, indexInBucket);
             } else {
                 //This should never happen when we picked a bucket for re-use
@@ -1455,6 +1457,8 @@ public:
         } else {
             putIntoFreeList(bucket, bucketPtr);
         }
+
+        Q_ASSERT(m_currentBucket < m_buckets.size());
     }
 
     using MyDynamicItem = DynamicItem<Item, markForReferenceCounting>;
@@ -1523,6 +1527,7 @@ public:
             bucketPtr = m_buckets.at(bucket);
         }
         unsigned short indexInBucket = index & 0xffff;
+        Q_ASSERT(m_currentBucket < m_buckets.size());
         return bucketPtr->itemFromIndex(indexInBucket);
     }
 
@@ -1589,6 +1594,7 @@ public:
 
     Statistics statistics() const
     {
+        Q_ASSERT(m_currentBucket < m_buckets.size());
         Statistics ret;
         uint loadedBuckets = 0;
         for (auto* bucket : m_buckets) {
