@@ -1,23 +1,8 @@
 /*
- * Copyright 2014 Kevin Funk <kfunk@kde.org>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
+    SPDX-FileCopyrightText: 2014 Kevin Funk <kfunk@kde.org>
+
+    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+*/
 
 #include "noprojectcustomincludepaths.h"
 
@@ -36,7 +21,7 @@ NoProjectCustomIncludePaths::NoProjectCustomIncludePaths(QWidget* parent)
     m_ui->setupUi(this);
     m_ui->storageDirectory->setMode(KFile::Directory);
 
-    setWindowTitle(i18n("Setup Custom Include Paths"));
+    setWindowTitle(i18nc("@title:window", "Setup Custom Include Paths"));
 
     connect(m_ui->directorySelector, &QPushButton::clicked, this, &NoProjectCustomIncludePaths::openAddIncludeDirectoryDialog);
 }
@@ -59,7 +44,11 @@ void NoProjectCustomIncludePaths::appendCustomIncludePath(const QString& path)
 QStringList NoProjectCustomIncludePaths::customIncludePaths() const
 {
     const QString pathsText = m_ui->customIncludePaths->document()->toPlainText();
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+    const QStringList paths = pathsText.split(QLatin1Char('\n'), Qt::SkipEmptyParts);
+#else
     const QStringList paths = pathsText.split(QLatin1Char('\n'), QString::SkipEmptyParts);
+#endif
     return paths;
 }
 
@@ -70,7 +59,7 @@ void NoProjectCustomIncludePaths::setCustomIncludePaths(const QStringList& paths
 
 void NoProjectCustomIncludePaths::openAddIncludeDirectoryDialog()
 {
-    const QString dirName = QFileDialog::getExistingDirectory(this, i18n("Select directory to include"));
+    const QString dirName = QFileDialog::getExistingDirectory(this, i18nc("@title:window", "Select Directory to Include"));
     if (dirName.isEmpty())
         return;
 
