@@ -11,6 +11,7 @@
 #include "test_perforce.h"
 
 #include <QTest>
+#include <QDir>
 #include <QDirIterator>
 #include <QStandardPaths>
 
@@ -25,19 +26,20 @@
 #define VERIFYJOB(j) \
     QVERIFY(j); QVERIFY(j->exec()); QVERIFY((j)->status() == VcsJob::JobSucceeded)
 
-const QString tempDir = QDir::tempPath();
-const QString perforceTestBaseDirNoSlash(tempDir + "/kdevPerforce_testdir");
-const QString perforceTestBaseDir(tempDir + "/kdevPerforce_testdir/");
-const QString perforceTestBaseDir2(tempDir + "/kdevPerforce_testdir2/");
-const QString perforceConfigFileName(QStringLiteral("p4config.txt"));
-
-const QString perforceSrcDir(perforceTestBaseDir + "src/");
-const QString perforceTest_FileName(QStringLiteral("testfile"));
-const QString perforceTest_FileName2(QStringLiteral("foo"));
-const QString perforceTest_FileName3(QStringLiteral("bar"));
-
 using namespace KDevelop;
 
+PerforcePluginTest::PerforcePluginTest()
+    : tempDir{QDir::tempPath()}
+    , perforceTestBaseDirNoSlash{tempDir + QLatin1String("/kdevPerforce_testdir")}
+    , perforceTestBaseDir{perforceTestBaseDirNoSlash + QLatin1Char('/')}
+    , perforceConfigFileName{QStringLiteral("p4config.txt")}
+    , perforceSrcDir{perforceTestBaseDir + QLatin1String("src/")}
+    , perforceTest_FileName{QStringLiteral("testfile")}
+    , perforceTest_FileName2{QStringLiteral("foo")}
+    , perforceTest_FileName3{QStringLiteral("bar")}
+{
+}
+
 void PerforcePluginTest::initTestCase()
 {
     AutoTestShell::init({QStringLiteral("kdevperforce")});
@@ -94,9 +96,7 @@ void PerforcePluginTest::createNewTempDirs()
     }
     g.close();
 
-
     tmpdir.mkdir(perforceSrcDir);
-    tmpdir.mkdir(perforceTestBaseDir2);
 }
 
 
@@ -105,10 +105,6 @@ void PerforcePluginTest::removeTempDirsIfAny()
     QDir dir(perforceTestBaseDir);
     if (dir.exists() && !dir.removeRecursively())
         qDebug() << "QDir::removeRecursively(" << perforceTestBaseDir << ") returned false";
-
-    QDir dir2(perforceTestBaseDir);
-    if (dir2.exists() && !dir2.removeRecursively())
-        qDebug() << "QDir::removeRecursively(" << perforceTestBaseDir2 << ") returned false";
 }
 
 
